==> cirq_unstable-0.9.0.dev20200721210651-py3-none-any.whl: cirq/work/zeros_sampler_test.py <==
# Copyright 2020 The Cirq Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
import pytest
import sympy

import cirq
from cirq import study
from cirq.google.common_serializers import (SINGLE_QUBIT_SERIALIZERS,
                                            SINGLE_QUBIT_DESERIALIZERS)
from cirq.google.serializable_gate_set import SerializableGateSet


def test_run_sweep():
    a, b, c = [cirq.NamedQubit(s) for s in ['a', 'b', 'c']]
    circuit = cirq.Circuit([cirq.measure(a)], [cirq.measure(b, c)])
    sampler = cirq.ZerosSampler()

    result = sampler.run_sweep(circuit, None, 3)

    assert len(result) == 1
    assert result[0].measurements.keys() == {'a', 'b,c'}
    assert result[0].measurements['a'].shape == (3, 1)
    assert np.all(result[0].measurements['a'] == 0)
    assert result[0].measurements['b,c'].shape == (3, 2)
    assert np.all(result[0].measurements['b,c'] == 0)


def test_sample():
    # Create a circuit whose measurements are always zeros, and check that
    # results of ZeroSampler on this circuit are identical to results of
    # actual simulation.
    qs = cirq.LineQubit.range(6)
    c = cirq.Circuit([cirq.CNOT(qs[0], qs[1]), cirq.X(qs[2]), cirq.X(qs[2])])
    c += cirq.Z(qs[3])**sympy.Symbol('p')
    c += [cirq.measure(q) for q in qs[0:3]]
    c += cirq.measure(qs[4], qs[5])
    # Z to even power is an identity.
    params = study.Points(sympy.Symbol('p'), [0, 2, 4, 6])

    result1 = cirq.ZerosSampler().sample(c, repetitions=10,
                                         params=params).sort_index(axis=1)
    result2 = cirq.Simulator().sample(c, repetitions=10,
                                      params=params).sort_index(axis=1)

    assert np.all(result1 == result2)


def test_sample_with_gate_set():
    gate_set = SerializableGateSet('test', SINGLE_QUBIT_SERIALIZERS,
                                   SINGLE_QUBIT_DESERIALIZERS)
    sampler = cirq.ZerosSampler(gate_set=gate_set)
    a, b = cirq.LineQubit.range(2)
    circuit1 = cirq.Circuit([cirq.X(a)])
    circuit2 = cirq.Circuit([cirq.CX(a, b)])

    sampler.sample(circuit1)

    with pytest.raises(AssertionError, match='Unsupported operation'):
        sampler.sample(circuit2)

==> cirq_unstable-0.10.0.dev20201020034551-py3-none-any.whl: cirq/work/zeros_sampler_test.py <==
# Copyright 2020 The Cirq Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
import pytest
import sympy

import cirq


def test_run_sweep():
    a, b, c = [cirq.NamedQubit(s) for s in ['a', 'b', 'c']]
    circuit = cirq.Circuit([cirq.measure(a)], [cirq.measure(b, c)])
    sampler = cirq.ZerosSampler()

    result = sampler.run_sweep(circuit, None, 3)

    assert len(result) == 1
    assert result[0].measurements.keys() == {'a', 'b,c'}
    assert result[0].measurements['a'].shape == (3, 1)
    assert np.all(result[0].measurements['a'] == 0)
    assert result[0].measurements['b,c'].shape == (3, 2)
    assert np.all(result[0].measurements['b,c'] == 0)


def test_sample():
    # Create a circuit whose measurements are always zeros, and check that
    # results of ZeroSampler on this circuit are identical to results of
    # actual simulation.
    qs = cirq.LineQubit.range(6)
    c = cirq.Circuit([cirq.CNOT(qs[0], qs[1]), cirq.X(qs[2]), cirq.X(qs[2])])
    c += cirq.Z(qs[3])**sympy.Symbol('p')
    c += [cirq.measure(q) for q in qs[0:3]]
    c += cirq.measure(qs[4], qs[5])
    # Z to even power is an identity.
    params = cirq.Points(sympy.Symbol('p'), [0, 2, 4, 6])

    result1 = cirq.ZerosSampler().sample(c, repetitions=10, params=params)
    result2 = cirq.Simulator().sample(c, repetitions=10, params=params)

    assert np.all(result1 == result2)


class OnlyMeasurementsDevice(cirq.Device):

    def validate_operation(self, operation: 'cirq.Operation') -> None:
        if not cirq.is_measurement(operation):
            raise ValueError(f'{operation} is not a measurement and this '
                             f'device only measures!')


def test_validate_device():
    device = OnlyMeasurementsDevice()
    sampler = cirq.ZerosSampler(device)

    a, b, c = [cirq.NamedQubit(s) for s in ['a', 'b', 'c']]
    circuit = cirq.Circuit(cirq.measure(a), cirq.measure(b, c))

    _ = sampler.run_sweep(circuit, None, 3)

    circuit = cirq.Circuit(cirq.measure(a), cirq.X(b))
    with pytest.raises(ValueError, match=r'X\(b\) is not a measurement'):
        _ = sampler.run_sweep(circuit, None, 3)
